/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|


  Bidouilleur STM32 Mini - exemple de contrôle de DAC LTC1661 via le port SPI.
  Électro-Bidouilleur, Juin 2017
  
    Définition des broches du SPI_1 (le premier SPI) dans la librairie SPI.h: 
    SS    <-->  PA4 <-->  BOARD_SPI1_NSS_PIN
    SCK   <-->  PA5 <-->  BOARD_SPI1_SCK_PIN
    MISO  <-->  PA6 <-->  BOARD_SPI1_MISO_PIN
    MOSI  <-->  PA7 <-->  BOARD_SPI1_MOSI_PIN

    DAC LTC1661: 
    Mot de 16 bits pour mise à jour du DAC A:
     1001ddddddddddxx en binaire

      1001: Met à jour le DAC A
      dddddddddd: La valeur de 10 bits du DAC A
      xx: Peut être n'importe quoi, par exemple 00

    Le LTC1661 possède un bus uni-directionnel (pas de possibilité de lecture par SPI)
*/

#include <SPI.h>
#define SPI1_NSS_PIN PA4    //SPI_1 La broche du signal SS est assignée à PA4.

void sendSPI(unsigned int tx_data)
{
  unsigned int rx_data1;    // registres de réception des permiers 8 bits de l'esclave
  unsigned int rx_data2;    // registres de réception des derniers 8 bits de l'esclave

  digitalWrite(SPI1_NSS_PIN, LOW); // abaissement de la ligne SS 
  rx_data1 = SPI.transfer(tx_data >> 8); // Envoi des premiers 8 bits sur SPI-1. Rien n'est retourné par le DAC dans rx_data
  rx_data2 = SPI.transfer(tx_data & 0xFF); // Envoi des derniers 8 bits sur SPI-1. Rien n'est retourné par le DAC dans rx_data
  digitalWrite(SPI1_NSS_PIN, HIGH); // élévation de la ligne SS
}

void setup()   // La fonction de configuration
{
  // Setup SPI 1: Voir la documentation en ligne de "Arduino SPI library"
  SPI.begin(); //Initialisation du port SPI_1
  SPI.setBitOrder(MSBFIRST); // Ordre des bits: MSB en premier
  SPI.setDataMode(SPI_MODE0); // SPI_1 data mode à 0
  SPI.setClockDivider(SPI_CLOCK_DIV16);    // Ajustement de la vitesse du bus (72 / 16 = 4.5 MHz)
  pinMode(SPI1_NSS_PIN, OUTPUT);  // Mise en mode sortie de la broche SS
}

void loop()   // La boucle sans fin
{
  unsigned int dac_val;  // variable locale de la valeur du DAC

  for (dac_val = 450 ; dac_val < 1024 ; dac_val++)  // balayage du DAC de 450 à 1023, approprié pour l'illumination de la DEL
  {
    sendSPI((0b1001<<12) + (dac_val<<2));   // 1001ddddddddddxx en binaire. Voir entête plus haut
    delayMicroseconds(1000);    //Délai de 1 milli-seconde pour ralentir le balayage des valeurs du DAC.
  }
  for (dac_val = 1023 ; dac_val > 450 ; dac_val--) // balayage du DAC de 1023 à 450, approprié pour l'illumination de la DEL
  {
    sendSPI((0b1001<<12) + (dac_val<<2));   // 1001ddddddddddxx en binaire. Voir entête plus haut
    delayMicroseconds(1000);    //Délai de 1 milli-seconde pour ralentir le balayage des valeurs du DAC.
  }
}

